/**
 * Created by Grigorii Shovkoplias on 20.11.2015.
 */

import java.io.*;
import java.util.*;

public class TestGen {

    private void gen() throws Exception {
        new File("../tests").mkdir();
        done = 0;
        genSamples();
        genSimples();
        genRand(10, 10);
        genSame(7, 7);
        genSort(13, 5);

        for (int i = 10; i <= MAXN; i *= 10) {
            genRand(gand(i), MAXX);
            genSort(gand(i), MAXX);
            genRand(gand(i), MAXX);
            genSame(gand(i), rand(MAXX) + 1);
        }

        genRand(MAXN, MAXX);
        genSame(MAXN, MAXX);
        genSort(MAXN, MAXX);


    }

    public static void main(String[] args) throws Exception {
        new TestGen().gen();
    }

    private int done = 0;
    private final Random rnd = new Random(3289214075201L);
    private final int MAXN = 100_000;
    private final int MAXX = 1000;

    private int rand(int n) {
        if (n < 1)
            return 0;
        return rnd.nextInt(n);
    }

    private int gand(int n) {
        return rand(n / 2) + n / 2 + 1;
    }

    private char zeroOrOne() {
        return (char) ('0' + rnd.nextInt(2));
    }

    private long rand(long n) {
        if (n < 1)
            return 0;
        long res = rand(Integer.MAX_VALUE);
        res <<= 32;
        res += rand(Integer.MAX_VALUE);
        res %= n;
        return res;
    }

    private void print(int[] a) throws IOException {
        done++;
        System.err.println("Generation test #" + done);
        PrintWriter out = new PrintWriter(String.format("../tests/%02d", done));
        out.println(a.length);
        for (int i = 0; i < a.length - 1; i++) {
            out.print(a[i] + " ");
        }
        out.println(a[a.length - 1]);

        out.close();
    }

    //begin generators

    private void genSamples() throws IOException {
        print(new int[]{1, 2, 3, 4});
        print(new int[]{1, 2, 1, 1});
        print(new int[]{1, 1, 2, 2, 3, 3});
    }

    private void genSimples() throws IOException {
        print(new int[]{1, 1, 1});
        print(new int[]{1, 1, 1, 1});
        print(new int[]{1, 2, 2, 2, 2, 3});
    }

    private void genRand(int n, int maxn) throws IOException {
        int a[] = new int[n];
        for (int i = 0; i < n; i++) {
            a[i] = rand(maxn) + 1;
        }
        print(a);
    }

    private void genSame(int n, int x) throws IOException {
        int a[] = new int[n];
        for (int i = 0; i < n; i++) {
            a[i] = x;
        }
        print(a);
    }

    private void genSort(int n, int maxn) throws IOException {
        int a[] = new int[n];
        for (int i = 0; i < n; i++) {
            a[i] = rand(maxn) + 1;
        }
        Arrays.sort(a);
        print(a);
    }



    //end generators

}
