import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Random;

public class TestGen {
    Random random = new Random(0xBEE);

    int testCount;
    public static final int MAX_N = 1000_00;

    void print(int n, int[] l, int[] r) throws IOException {
        if (l.length > MAX_N)
            throw new IllegalArgumentException("Too long array");

        List<Integer> p = new ArrayList<>();
        for (int i = 0; i < l.length; ++i) p.add(i);
        Collections.shuffle(p, random);

        PrintWriter out = new PrintWriter(String.format("../tests/%02d", ++testCount));
        System.out.println("Generating test #" + testCount);
        out.println(n + " " + l.length);
        for (int i = 0; i < l.length; ++i)
            out.println((l[p.get(i)] + 1) + " " + (r[p.get(i)] + 1));
        out.close();
    }

    void genSamples() throws IOException {
        print(3, new int[] {1}, new int[]{2});
        print(6, new int[] {1, 2, 3}, new int[]{2, 5, 5});
        print(5, new int[] {1, 2, 3}, new int[]{1, 2, 4});
    }

    void run() throws IOException {
        new File("../tests").mkdir();
        genSamples();
        genHandleTest();
        genShift(MAX_N, MAX_N / 2, 1);
        genShift(MAX_N, MAX_N / 2, 10);
        genShift(MAX_N, MAX_N / 2, 100);
        genShift(MAX_N, MAX_N / 2, 1000);
        genFixedLen(MAX_N, MAX_N, MAX_N / 2);
        genFixedLen(MAX_N, MAX_N, (int)Math.sqrt(MAX_N));
        genFixedLen(MAX_N, 20, MAX_N - 10);
        genFixedLen(MAX_N, MAX_N, 100);
        genFixedLen(MAX_N, MAX_N, 5);
        for (int i = 0; i < 10; ++i)
            genRandom(MAX_N, MAX_N);
        int[] LENS = new int[]{1, 2, 3, 5, 10, 20, 25, 50, 100, 150, 200};
        for (int LEN : LENS)
            genRandomMaxLen(MAX_N, MAX_N, LEN);
    }

    private void genHandleTest() throws IOException {
        int[] l = new int[MAX_N];
        int[] r = new int[MAX_N];
        for (int i = 0; i < MAX_N; ++i)
            l[i] = r[i] = i;
        print(MAX_N, l, r);
    }

    private void genRandomMaxLen(int n, int m, int len) throws IOException {
        int[] l = new int[m];
        int[] r = new int[m];
        for (int i = 0; i < m; ++i) {
            l[i] = random.nextInt(n);
            r[i] = l[i] + random.nextInt(Math.min(n - l[i], len));
        }
        print(n, l, r);
    }

    private void genFixedLen(int n, int m, int len) throws IOException {
        int[] l = new int[m];
        int[] r = new int[m];
        for (int i = 0; i < m; ++i) {
            l[i] = random.nextInt(n - len + 1);
            r[i] = l[i] + len - 1;
        }
        print(n, l, r);
    }

    private void genShift(int n, int len, int sh) throws IOException {
        List<Integer> l = new ArrayList<>();
        List<Integer> r = new ArrayList<>();
        int start = 0;
        while (start + len <= n) {
            l.add(start);
            r.add(start + len - 1);
            start += sh;
        }
        print(n, toArrInt(l), toArrInt(r));
    }

    int[] toArrInt(List<Integer> l) {
        int[] ret = new int[l.size()];
        for (int i = 0; i < ret.length; ++i)
            ret[i] = l.get(i);
        return ret;
    }


    void genRandom(int n, int m) throws IOException {
        int[] l = new int[m];
        int[] r = new int[m];
        for (int i = 0; i < m; ++i) {
            l[i] = random.nextInt(n);
            r[i] = random.nextInt(n);
            if (l[i] > r[i]) {
                int t = l[i];
                l[i] = r[i];
                r[i] = t;
            }
        }
        print(n, l, r);
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}
